#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2011 Async Open Source
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Author(s): Ronaldo Maia   <romaia@async.com.br>


import os
import struct

import gtk
import gobject
import pango

from kiwi.ui.delegates import GladeDelegate


(PP_OK,
 PP_PROCESSING,
 PP_NOTIFY) = range(3)

PP_CANCEL = 13

def crc16(buff, mask=0x1021):
    crc = 0
    for ch in buff:
        ch = ord(ch) << 8

        for i in range(8):
            if (crc ^ ch) & 0x8000 :
                crc = (crc << 1) ^ mask
            else:
                crc = crc << 1 & 0xffff
            ch <<= 1
    crc = crc & 0xffff
    return struct.pack('>H', crc)

def format_numero(numero):
    parts = []
    for i in range(0, 16, 4):
        parts.append(numero[i:i+4])

    return ' '.join(parts)

SYN = 0x16
ETB = 0x17
ACK = 0x06
PING = 0x18
PONG = 0x04

class Card(object):
    fields = (
        ('tipo', 2),
        ('status', 1),
        ('aplicacao', 2),
        ('rede', 2),
        ('registro', 2),
        ('tam_trilha1', 2),
        ('trilha1', 76),
        ('tam_trilha2', 2),
        ('trilha2', 37),
        ('tam_trilha3', 3),
        ('trilha3', 104),
        ('tam_pan', 2),
        ('pan', 19),
        ('application_pan', 2),
        ('application_label', 16),
        ('service_code', 3),
        ('cardholder_name', 26),
        ('expiration_date', 6),
        ('tam_numero', 2),
        ('numero', 19),
        ('saldo', 8),
        ('issuer_country', 3),
    )

    def __init__(self, string):
        self.string = string
        for (name, size) in self.fields:
            part = string[:size]
            string = string[size:]
            setattr(self, name, part)

    def get_description(self):
        if self.tipo == '00':
            start = self.trilha1.index('^') + 1
            end = self.trilha1.index('^', start)
            name = self.trilha1[start:end].strip()
            numero = format_numero(self.trilha1[1:start-1])
            return '%s' % (numero)
            return 'Magnetico - %s' % (numero)
        elif self.tipo == '03':
            name = self.cardholder_name.strip()
            return 'Chip - %s' % name




class PinPad:

    INFO = {
        '00': ( 'Ingenico do Brasil  '
                'I3500MMT003C        '
                'OS 2.5B SSA: 1.25   '
                '1.05'
                '001.16 - 061212 '
                '006952406731        '),
        '01': ( 'AMEX                '
                '001.15 051229'
                '       '
                '00'),
        '02': ( 'REDECARD            '
                '001.16 060508'
                '       '
                '00'),
        '03': ( 'VISANET             '
                '001.17 051229'
                '105g02 '
                '00'),
        '04': ( 'OTHER               '
                '001.05 061027'
                '       '
                '00'),
    }

    def __init__(self, port):
        self._file = open(port, 'w+b')
        self.master = self._file.fileno()

    #
    #   Private API
    #

    def _parse_params(self, string):
        params = []
        while string:
            size = int(string[:3])
            params.append(string[3:3+size])
            string = string[3+size:]
        return params

    def _handle(self, string):
        cmd = string[:3]
        if not hasattr(self, cmd):
            self.debug('unhandled command: %s' % cmd)
            return None

        method = getattr(self, cmd)
        assert callable(method), '%s should be callable' % cmd

        params = self._parse_params(string[3:])
        self.debug('Comand: %s - Params: %s' % (cmd, params))

        method(*params)

    def _handle_data(self, source=None, condition=None):
        d = os.read(self.master, 1)

        if ord(d) == PING:
            self.debug('ping')
            os.write(self.master, chr(PONG))

        # SYN Representa o inicio de um comando.
        # Depois de encontrado um SYN, os dados devem ser lidos até
        # encontrar um ETB, que representa o final de um comando. Depois
        # disso, ainda há dois caracteres de checksum (CRC16)
        elif ord(d) == SYN:
            cmd = ''
            while True:
                d = os.read(self.master, 1)
                if ord(d) == ETB:
                    cs = os.read(self.master, 2) # Dois CRC depois do ETB
                    break
                cmd += d

            os.write(self.master, chr(ACK))
            self._handle(cmd)
        else:
            self.debug('OOPS %s' % hex(ord(d)))

        return True

    def run(self):
        self.debug('Pinpad is running')
        while True:
            self._handle_data()

    def _merge_reply(self, cmd, retval):
        if not isinstance(retval, tuple):
            retval = (retval,)

        reply = '%s%03d' % (cmd, retval[0])
        for i in retval[1:]:
            reply += '%03d%s' % (len(i), i)
        self.debug('Reply: %s' % reply)
        return reply


    def _send_reply(self, cmd, *args):
        reply = self._merge_reply(cmd, args)

        # Dont know why, but there is an extra SYN at the end of
        # the reply.
        if cmd[:3] == 'ENB':
            reply = reply + chr(SYN)

        reply = reply + chr(ETB)
        checksum = crc16(reply)
        os.write(self.master, chr(SYN) + reply + checksum)

    #
    #   Overrideable Methods
    #

    def display(self, message):
        print '>>> %S <<<' % message

    def debug(self, message):
        print message

    #
    #   COMANDOS
    #

    def OPN(self):
        """PP_Open
        """
        self._send_reply('OPN', PP_OK)

    def CLO(self, message):
        """PP_Close
        """
        self.display(message)
        self._send_reply('CLO', PP_OK)

    def DSP(self, message):
        """PP_Display
        """
        self.display(message)
        self._send_reply('DSP', PP_OK)

    def DEX(self, message):
        """PP_DisplayEx
        """
        self.display(message)
        self._send_reply('DEX', PP_OK)

    def GKY(self, message):
        """PP_StartGetKey / PP_GetKey
        """
        raise NotImplementedError

    def GPN(self, data=None):
        """PP_StartGetPIN / PP_GetPIN

        Resposta:

        H16 PIN Criptografado
        H20 Numero de série da chave, somente no caso do DUKPT
        """

        data = ('0' * 16 +
                '0' * 20)
        self._send_reply('GPN', PP_OK, data)

    def RMC(self, message):
        """PP_StartRemoveCard / PP_RemoveCard
        """
        self._send_reply('RMC', PP_OK)

    def GEN(self, message):
        """PP_StartGenericCmd / PP_GenericCmd
        """
        raise NotImplementedError

    def CKE(self):
        """
        PP_StartCheckEvent / PP_CheckEvent

        """
        start = '161B4984243006797797^'
        nome = 'RONALDO F MAIA     '
        middle = '^12092060000000576000000               37'
        cartao = '4984243006797797'
        end = '=12092060000057690770000'
        white = ' ' * 104

        data = start + nome + middle + cartao + end + white

        self._send_reply('CKE', PP_OK, data)

    def GCR(self, data):
        """
        PP_StartGetCard / PP_GetCard / PP_ResumeGetCard

        Entrada:
            N2  Identificador da Rede: (00 = todas)
            N2  Tipo de aplicação (99 = Qualquer)
            N12 Valor inicial da transação (pode ser 0)
            N6  Data da transação
            N6  Hora da transação
            N10 Timestamp das tableas de parametros.
            N2  Quantidade de entradas a seguir (somente se aplicacao for
                00)
            ...


        Exemplo de entrada:
            00
            99
            000000000000
            110322
            165504
            0601201000
            00
        """

        data = CARD_1
        self._send_reply('GCR', PP_OK, data)

    def GOC(self, message):
        """PP_StartGoOnChip / PP_GoOnChip
        """
        raise NotImplementedError

    def FNC(self):
        """PP_FinishChip
        """
        raise NotImplementedError

    def GIN(self, param):
        """PP_GetInfo
        """
        data = self.INFO[param]
        self._send_reply('GIN', PP_OK, data)

    def ENB(self, data):
        """PP_EncryptBuffer

        Entrada:
            N1      Modo de Criptografia
            N2      Indice da Master Key
            H32     Working Key
            H15     Dados a serem criptografados

        Saida:
            H16     Dados criptografados
        """
        self._send_reply('ENB', PP_OK, '0' * 16)

    def TLI(self):
        """PP_TableLoadInit
        """
        raise NotImplementedError

    def TLR(self):
        """PP_TableLoadRec
        """
        raise NotImplementedError

    def TLE(self):
        """PP_TableLoadEnd
        """
        raise NotImplementedError

    def GDU(self, data):
        """PP_GetDUKPT
        """
        data = '0' * 20
        self._send_reply('GDU', PP_OK, data)

    def GTS(self):
        """PP_GetTimeStamp
        """
        raise NotImplementedError


class GtkPinPad(GladeDelegate, PinPad):

    def __init__(self, port, show_debug=True):
        GladeDelegate.__init__(self, gladefile="pinpad",
                               delete_handler=self.quit_if_last)
        self._setup_widgets()
        self.debug_frame.set_visible(show_debug)
        self.numpad_box.set_visible(show_debug)
        self.function_box.set_visible(show_debug)
        self.pinpad.set_deletable(show_debug)
        self.pinpad.set_resizable(show_debug)
        self.clean.set_visible(show_debug)
        if not show_debug:
            self.pinpad.set_size_request(220, -1)
        self.show()
        PinPad.__init__(self, port)

        gobject.io_add_watch(self._file, gobject.IO_IN, self._handle_data)
        self._current_blocker = None

    def _read_cards(self):
        cards_file = open('cards.txt')
        cards = []
        for line in cards_file.readlines():
            c = Card(line[:-1])
            cards.append((c.get_description(), c))
        self.card_combo.prefill(cards)
        data = self.card_combo.get_selected_data()

    def _setup_widgets(self):
        self.display_box.modify_bg(gtk.STATE_NORMAL,
                                   gtk.gdk.color_parse('#8caa7c'))

        self.line1.modify_font(pango.FontDescription("Monospace 15"))
        layout = self.line1.get_layout()
        layout.set_text('X'*16)
        width = layout.get_pixel_size()[0] * 2

        for i in (self.line1, self.line2):
            i.modify_font(pango.FontDescription("Monospace 15"))
            i.modify_fg(gtk.STATE_NORMAL,
                        gtk.gdk.color_parse('black'))
        self.display(('      Stoq      '
                      '     PinPad     '))
        self.buffer = self.debug_view.get_buffer()

        self._read_cards()
        self._disable_all_widgets()

        self.pinpad.move(0, 370)

    def _disable_all_widgets(self):
        # Disable every widget, until some action is required
        self.insert.set_sensitive(False)
        self.remove.set_sensitive(False)

        for w in range(10):
            btn = getattr(self, 'button%d' % w)
            btn.set_sensitive(False)

        for w in range(1, 5):
            btn = getattr(self, 'f%d' % w)
            btn.set_sensitive(False)

        self.cancel.set_sensitive(False)
        self.clean.set_sensitive(False)
        self.ok.set_sensitive(False)

    def debug(self, message):
        self.buffer.insert_at_cursor(message + '\n')
        #iter = self.buffer.get_end_iter()
        #self.debug_view.scroll_to_iter(iter, 0, True, 0, 0)

        mark = self.buffer.get_insert()
        self.debug_view.scroll_mark_onscreen(mark)

    def display(self, message):
        p1 = message[:16]
        p2 = message[16:]
        self.line1.set_text(p1)
        self.line2.set_text(p2)

    def GCR(self, data):
        self._current_blocker = 'GCR'
        msg = ('Insira ou Passe '
               '    o Cartão    ')
        self.display(msg)
        if self.automatic_mode.get_active():
            self._insert_card()
        else:
            self.insert.set_sensitive(True)
            self.cancel.set_sensitive(True)
            self._send_reply('NTM', PP_OK, msg)
            self.pinpad.set_keep_above(True)

    def GPN(self, data=None):
        """PP_StartGetPIN / PP_GetPIN
        """
        self._current_blocker = 'GPN'

        msg = data[61:93]
        self.display(msg)
        self._send_reply('NTM', PP_OK, msg)
        self._data = '7B78555F0BE5F6AE00000000000000000000'
        if self.automatic_mode.get_active():
            self._press_ok()
        else:
            self.ok.set_sensitive(True)
            self.cancel.set_sensitive(True)
            self.pinpad.set_keep_above(True)

    def RMC(self, message):
        """PP_StartRemoveCard / PP_RemoveCard
        """
        msg = message
        self.display(msg)
        self._send_reply('NTM', PP_OK, msg)
        if self.automatic_mode.get_active():
            self._remove_card()
        else:
            self.remove.set_sensitive(True)

    #
    #   Private
    #

    def _insert_card(self):
        data = self.card_combo.get_selected_data()
        self._send_reply('GCR', PP_OK, data.string)

    def _remove_card(self):
        self._send_reply('RMC', PP_OK)

    def _press_cancel(self):
        if not self._current_blocker:
            return

        self._send_reply(self._current_blocker, PP_CANCEL)
        self._current_blocker = None

    def _press_ok(self):
        if not self._current_blocker:
            return

        if self._data:
            self._send_reply(self._current_blocker, PP_OK, self._data)
        else:
            self._send_reply(self._current_blocker, PP_OK)
        self._current_blocker = None

    #
    #   CALLBACKS
    #

    def on_insert__clicked(self, button):
        self._insert_card()
        self.pinpad.set_keep_above(False)
        self.insert.set_sensitive(False)
        self.cancel.set_sensitive(False)

    def on_remove__clicked(self, button):
        self._remove_card()
        self.remove.set_sensitive(False)
        self.pinpad.set_keep_above(False)

    def on_cancel__clicked(self, widget):
        self._press_cancel()
        self._disable_all_widgets()
        self.pinpad.set_keep_above(False)

    def on_ok__clicked(self, widget):
        self._press_ok()
        self.ok.set_sensitive(False)
        self.cancel.set_sensitive(False)
        self.pinpad.set_keep_above(False)

import sys
def main():
    if len(sys.argv) == 1:
        print 'Usage: %s /dev/pts/x' % sys.argv[0]
        return

    port = sys.argv[-1]

    p = GtkPinPad(port, show_debug=False)
    gtk.main()
    #p.run()


if __name__ == '__main__':
    main()
