# -*- Mode: Python; coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

##
## Copyright (C) 2010 Async Open Source <http://www.async.com.br>
## All rights reserved
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., or visit: http://www.gnu.org/.
##
## Author(s):   George Kussumoto        <george@async.com.br>
##

import os.path
import subprocess

from kiwi.log import Logger

from stoqlib.database.orm import StringCol, IntCol
from stoqlib.domain.base import Domain
from stoqlib.lib.parameters import sysparam

import atexit

log = Logger('stoqlib.plugins.tef.tefdomain')

class TEFClient(Domain):
    """A TEF client domain class.
    @param path: The path where the TEF client JAR executable is located.
    """
    path = StringCol()
    host = StringCol(default='localhost')
    port = IntCol(default=60906)

    #
    # Public
    #

    def _kill(self):
        self.proc.kill()
        if hasattr(self, 'pinpad_proc'):
            self.pinpad_proc.kill()

    def _run_pinpad(self):
        pwd = os.path.dirname(__file__)
        pinpad_bin = os.path.join(pwd, 'pinpad', 'pinpad.py')
        cmd = ('%(bin)s /dev/ttyStty0' % dict(bin=pinpad_bin))
        self.pinpad_proc = subprocess.Popen(cmd.split(' '), shell=False,
                                cwd=os.path.join(pwd, 'pinpad'),
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)

    def run(self):
        java_bin = '/VeSPague/Java/bin/java'
        if not os.path.exists(java_bin):
            java_bin = '/usr/bin/java'

        if sysparam(self.get_connection()).DEMO_MODE:
            if not os.path.exists('/dev/ttyStty0'):
                module = os.path.join(os.path.dirname(__file__),
                                      'tty0tty.ko',)
                os.popen('sudo insmod %s' % module)
            path = os.path.join(os.path.dirname(__file__),
                                'ClientSimulator',
                                'simulador-com-rxtx.jar')
            cmd = ('%(bin)s -Dgnu.io.rxtx.SerialPorts=/dev/ttyStty1 '
                   '-Djava.library.path=/usr/lib/jni/ '
                   '-jar %(tefclient)s /dev/ttyStty1' % dict(
                bin=java_bin, tefclient=path))
            cwd = os.path.dirname(path)
            self._run_pinpad()
        else:
            cmd = '%(bin)s -jar %(tefclient)s %(host)s %(port)d' % dict(
                bin=java_bin, tefclient=self.path, host=self.host, port=self.port)
            cwd = os.path.dirname(self.path)
        log.debug('executing %s' % cmd)

        # The configuration file usally stays in the same directory as the
        # executable.
        self.proc = subprocess.Popen(cmd.split(' '), shell=False, cwd=cwd,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
        atexit.register(self._kill)
        return self.proc
