# -*- Mode: Python; coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

##
## Copyright (C) 2010 Async Open Source <http://www.async.com.br>
## All rights reserved
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., or visit: http://www.gnu.org/.
##
## Author(s):   George Kussumoto    <george@async.com.br>
##

import os
import sys
import gettext

from kiwi.component import get_utility
from kiwi.environ import environ
from zope.interface import implements

from stoqlib.database.migration import PluginSchemaMigration
from stoqlib.lib.interfaces import IPlugin, IPluginManager
from stoqlib.lib.pluginmanager import register_plugin
from stoqlib.lib.message import info
from stoq import library

plugin_root = os.path.dirname(__file__)
sys.path.append(plugin_root)
from tefui import TEFUI
from teftranslation import tef_gettext as _


class TEFPlugin(object):
    implements(IPlugin)
    name = 'tef'
    has_product_slave = False

    def __init__(self):
        self.ui = None
        locale_dir = None
        if library.uninstalled:
            path = os.path.dirname(__file__)
            locale_dir = os.path.join(path, 'locale')
        gettext.bindtextdomain('stoqtef', locale_dir)

    def get_migration(self):
        environ.add_resource('tefsql', os.path.join(plugin_root, 'sql'))
        return PluginSchemaMigration(self.name, 'tefsql', ['*.sql'])

    def get_tables(self):
        return [('tefdomain', ['TEFClient', ])]

    def activate(self):
        #FIXME: add our own java installation in the path.
        java = os.path.join(plugin_root, 'client/Java/bin')
        os.environ['PATH'] = ':'.join([java, os.environ['PATH']])

        # Set the LANG/LC_MONETARY, since the TEF plugin rely on the locale
        # to properly set the values in the pinpad.
        os.environ['LANG'] = 'pt_BR.UTF8'
        os.environ['LANGUAGE'] = 'pt_BR.UTF8'
        os.environ['LC_MONETARY'] = 'pt_BR.UTF8'

        manager = get_utility(IPluginManager)
        if manager.is_active('ecf'):
            environ.add_resource('glade', os.path.join(plugin_root, 'glade'))
            self.ui = TEFUI()
        else:
            info(_(u'Plugin ECF should be active to enable the TEF plugin.'))


register_plugin(TEFPlugin)
