#!/bin/bash

ROOT=$(dirname $0)
echo $ROOT
PREFIX=$ROOT/prefix
MODULE=file://`pwd`/jhbuild/stoq.modules
JHBUILDRC=$ROOT/jhbuild/stoq.jhbuildrc

# Development environment requires (feisty as a reference)
#
# wget gcc make gettext automake autoconf
# libxml-parser-perl zlib1g-dev libc6-dev python-dev
# python-crypto python-paramiko patch g++ unzip curl libssl-dev
# libcurl3-openssldev libexpat-dev docbook-xml docbook-xsl
# libpng12-dev flex bison libtiff4-dev tcl8.4 gperf
# libX11-dev libXext-dev libXrender-dev libXcomposite-dev libXt-dev
#
# Via tarball
# bzr 2.3.1 https://launchpad.net/bzr/2.3/2.3.1/+download/bzr-2.3.1.tar.gz
# git https://nodeload.github.com/git/git/zipball/v1.7.10
# xz http://tukaani.org/xz/xz-5.0.3.tar.gz
# zlib 1.2.7 http://zlib.net/zlib/zlib-1.2.7.tar.gz
# icu
#
# FROM bzr:
# intltool
#
# FROM git:
# jhbuild + non-root patch
# gnome-common
#

function bootstrap() {
    jhbuild -f $JHBUILDRC -m $MODULE buildone python

    export PYTHON=$PREFIX/bin/python
    _python jhbuild/ez_setup.py

    _python $PREFIX/bin/easy_install pyserial reportlab python-dateutil Mako xlwt
    mkdir -p prefix/_jhbuild/root-expat-broken
    mkdir -p prefix/_jhbuild/root-glib-broken
    mkdir -p prefix/_jhbuild/root-gobject-introspection-broken
    jhbuild -f $JHBUILDRC -m $MODULE buildone autoconf libtool automake pkg-config gettext

    _jhbuild build stoq
}

function _jhbuild() {
    PYTHON=$PREFIX/bin/python jhbuild -f $JHBUILDRC -m $MODULE "$@"
}

function shell() {
    _jhbuild shell
}

function run() {
    _jhbuild run "$@"
}

function build() {
    _jhbuild build "$@"
}

function list() {
    _jhbuild list "$@"
}

function buildone() {
    _jhbuild buildone "$@"
}

function cleanone() {
    _jhbuild cleanone "$@"
}

function _python() {
    PYTHON=$PREFIX/bin/python _jhbuild run python "$@"
}

function stoq() {
    _jhbuild run python $PREFIX/bin/stoq "$@"
}

function release() {
    VERSION=`cd checkout/stoq && python -c "import stoq; print stoq.version"`
    PKG_VERSION=1
    DEST=stoq-linux-${VERSION}-${PKG_VERSION}_x86
    rm -fr $DEST
    mkdir -p $DEST
    echo "* Copying files to $DEST"
    rsync -alr \
        -f"+ bin" \
        -f"+ bin/gdk-pixbuf-query-loaders" \
        -f"+ bin/python" \
        -f"+ bin/python2" \
        -f"+ bin/python2.?" \
        -f"+ bin/psql" \
        -f"+ bin/stoq*" \
        -f"+ etc" \
        -f"+ etc/fonts" \
        -f"+ etc/fonts/fonts.conf" \
        -f"+ etc/stoq" \
        -f"+ include/" \
        -f"+ include/python2.7/" \
        -f"+ include/python2.7/pyconfig.h" \
        -f"+ lib/" \
        -f"+ lib/***so***" \
        -f"+ lib/gdk-pixbuf-2.0" \
        -f"+ lib/gdk-pixbuf-2.0/2.10.0" \
        -f"+ lib/gdk-pixbuf-2.0/2.10.0/loaders" \
        -f"+ lib/gtk-2.0" \
        -f"+ lib/gtk-2.0/2.10.0" \
        -f"+ lib/gtk-2.0/2.10.0/engines" \
        -f"+ lib/python2.7" \
        -f"- lib/python2.7/*/test/***" \
        -f"- lib/python2.7/*/tests/***" \
        -f"- lib/python2.7/site-packages/stoq/tests/***" \
        -f"- lib/python2.7/site-packages/stoqlib/test/***" \
        -f"- lib/python2.7/site-packages/stoqlib/*/test/***" \
        -f"- lib/python2.7/site-packages/stoqlib/*/*/test/***" \
        -f"- lib/python2.7/site-packages/twisted/test/***" \
        -f"- lib/python2.7/site-packages/twisted/*/test/***" \
        -f"- lib/python2.7/site-packages/twisted/*/*/test/***" \
        -f"- lib/python2.7/site-packages/zope.interface/tests/***" \
        -f"- lib/python2.7/test/***" \
        -f"+ lib/python2.7/***" \
        -f"+ lib/stoqlib" \
        -f"+ lib/stoqlib/***" \
        -f"+ share" \
        -f"+ share/doc" \
        -f"+ share/doc/stoq" \
        -f"+ share/doc/stoq/***" \
        -f"+ share/icons/***" \
        -f"+ share/mime/***" \
        -f"+ share/locale" \
        -f"+ share/locale/pt_BR" \
        -f"+ share/locale/pt_BR/***" \
        -f"+ share/polkit-1" \
        -f"+ share/stoq***" \
        -f"+ share/themes/***" \
        -f"- ***.la" \
        -f"- ***.py" \
        -f"- *" \
        prefix/ $DEST/

    rm $DEST/lib/libpng.so*
    cp run-stoq.sh $DEST
    tar cf $DEST.tar $DEST
    xz -c $DEST.tar > $DEST.tar.xz
    rm $DEST.tar
}

function usage() {
    echo $"Usage: $0 [command]"
    echo "List of commands:"
    echo
    declare -F|grep -v chroot_|egrep -v "^declare -f _"|cut -d\  -f3-|sed "s/^/    /"
    echo
}

if [ "$(declare -F | egrep ^declare\ -f\ $1\$)" = "" ]; then
    echo "ERROR: no such command: $1"
    usage
    exit 1
fi

unset PYTHONPATH
unset PYTHON
unset LC_ALL LC_CTYPE LANG LANGUAGE LC_MONETARY LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT LC_IDENTIFICATION LC_NUMERIC LC_PAPER LC_TIME LC_NAME

set -x -e

$@

exit 0
